/* $Id$
   
   Copyright (C) 2018-2021 Aamot Software
   Author(s): Ole Aamot <ole@gnome.org>
   License: GNU GPL version 3
   Version: 4.0.0 (2022-04-30)
   Website: http://www.gingerblue.org/
   
*/

GtkWidget *line (gint jack, gchar *label);
